#ifndef __UTIL_H__
#define __UTIL_H__

#include <ogg/ogg.h>
#include "ogmmerge.h"

/*
 * mymalloc & myfree for debugging purposes
 */

#if 0
  #define mymalloc(s) _mymalloc(s, __FILE__, __LINE__)
  #define myfree(p) _myfree(p, __FILE__, __LINE__)
#else
  #define mymalloc(s) malloc(s)
  #define myfree(p) free(p)
#endif

char *_mymalloc(int size, char *name, int line);
void _myfree(void *p, char *name, int line);

#define die(a) _die(a, __FILE__, __LINE__)
void _die(const char *, const char *, int);

/*
 * Packet queueing stuff
 */

/* Callback function used to calculate the page's timestamp */ 
typedef u_int64_t (*q_make_timestamp_t)(ogmmerge_state_t *, ogg_int64_t);

/* q_page_t is used internally only */
typedef struct q_page {
  ogmmerge_page_t *mpage;
  struct q_page   *next;
} q_page_t;

typedef struct q_t {
  struct q_page      *first;
  struct q_page      *current;
  q_make_timestamp_t  make_timestamp_f;
  ogmmerge_state_t   *private;
  ogg_stream_state   *os;
} q_t;

q_t *q_init(ogmmerge_state_t *, ogg_stream_state *, q_make_timestamp_t);
void q_done(q_t *);

int q_add_ogg_page(q_t *, ogg_page *);
int q_flush_pages(q_t *q);
int q_queue_pages(q_t *q);
ogmmerge_page_t *q_page_out(q_t *q);
int q_page_available(q_t *q);

int pq_init();
int pq_done();
int pq_add_ogmmerge_page(ogmmerge_page_t *);
ogmmerge_page_t *pq_page_out();


#endif  /* __UTIL_H__ */
